@@ -102,6 +102,22 @@ class TestManifest(BaseTest):
 			'D /Dir'],
 			list(manifest.generate_manifest(self.tmpdir, alg='sha256')))
 
+	def testParseManifest(self):
+		self.assertEquals({}, manifest._parse_manifest(''))
+		parsed = manifest._parse_manifest('F e3d5983c3dfd415af24772b48276d16122fe5a87 1172429666 2980 README\n'
+						  'X 8a1f3c5f416f0e63140928102c44cd16ec2c6100 1172429666 5816 install.sh\n'
+						  'D /0install\n'
+						  'S 2b37e4457a1a38cfab89391ce1bfbe4dc5473fc3 26 mime-application:x-java-archive.png\n')
+		keys = parsed.keys()
+		keys.sort()
+		assert keys == ['0install', '0install/mime-application:x-java-archive.png', 'README', 'install.sh']
+		for bad in ['Hello', 'D bob\n', 'D /bob\nD /bob\n']:
+			try:
+				manifest._parse_manifest(bad)
+				assert False
+			except BadDigest:
+				pass
+
 suite = unittest.makeSuite(TestManifest)
 if __name__ == '__main__':
 	sys.argv.append('-v')
