@@ -14,7 +14,7 @@ class Dialog(gtk.Dialog):
 
 		def destroyed(widget):
 			if self.__shown:
-				_one_less_window()
+				one_less_window()
 		self.connect('destroy', destroyed)
 
 	def show(self):
@@ -43,13 +43,13 @@ def alert(parent, message, type = gtk.MESSAGE_ERROR):
 				str(message))
 	box.set_position(gtk.WIN_POS_CENTER)
 	def resp(b, r):
-		box.destroy()
-		_one_less_window()
+		b.destroy()
+		one_less_window()
 	box.connect('response', resp)
 	box.show()
 	n_windows += 1
 
-def _one_less_window():
+def one_less_window():
 	global n_windows
 	n_windows -= 1
 	if n_windows == 0:
@@ -86,6 +86,7 @@ def frame(page, title, content, expand = False):
 	if type(content) in (str, unicode):
 		content = gtk.Label(content)
 		content.set_alignment(0, 0.5)
+		content.set_selectable(True)
 	frame.add(content)
 	if hasattr(content, 'set_padding'):
 		content.set_padding(8, 4)
