@@ -1,4 +1,5 @@
 import gtk, os, gobject, sys
+import gtk.glade
 
 from zeroinstall.injector.iface_cache import iface_cache
 from zeroinstall.injector.policy import Policy
@@ -8,11 +9,26 @@ from zeroinstall.injector.reader import InvalidInterface
 import dialog
 from checking import CheckingBox
 
-version = '0.29'
+version = '0.31'
 
 # Singleton Policy
 policy = None
 
+gladefile = os.path.join(os.path.dirname(__file__), 'zero-install.glade')
+
+# Wrapped for glade widget tree that throws a sensible exception if the widget isn't found
+class Template:
+	def __init__(self, root):
+		self.widgets = gtk.glade.XML(gladefile, root)
+		self.root = root
+	
+	def get_widget(self, name = None):
+		if not name:
+			name = self.root
+		widget = self.widgets.get_widget(name)
+		assert widget, "Widget '%s' not found in glade file '%s'" % (name, gladefile)
+		return widget
+
 class GUIHandler(object):
 	monitored_downloads = None
 	dl_callbacks = None		# Download -> [ callback ]
@@ -73,15 +89,15 @@ class GUIHandler(object):
 						if self.policy.window:
 							self.policy.window.browser.build_tree()
 				except download.DownloadError, ex:
-					dialog.alert(self.policy.window,
+					dialog.alert(self.policy.window.window,
 						"Error downloading '%s':\n\n%s" %
 						(name, ex))
 				except InvalidInterface, ex:
-					dialog.alert(self.policy.window,
+					dialog.alert(self.policy.window.window,
 						"Syntax error in downloaded interface '%s':\n\n%s" %
 						(name, ex))
 				except SafeException, ex:
-					dialog.alert(self.policy.window,
+					dialog.alert(self.policy.window.window,
 						"Error fetching '%s':\n\n%s" %
 						(name, ex))
 
@@ -110,7 +126,7 @@ class GUIHandler(object):
 
 	def confirm_trust_keys(self, interface, sigs, iface_xml):
 		import trust_box
-		trust_box.confirm_trust(interface, sigs, iface_xml)
+		trust_box.confirm_trust(interface, sigs, iface_xml, parent = self.policy.checking or self.policy.window.window)
 	
 	def report_error(self, ex):
 		dialog.alert(None, str(ex))
@@ -120,6 +136,7 @@ class GUIPolicy(Policy):
 	checking = None		# GtkDialog ("Checking for updates...")
 	original_implementation = None
 	download_only = None
+	widgets = None		# Glade
 
 	def __init__(self, interface, download_only, refresh, src = False, restrictions = None):
 		Policy.__init__(self, interface, GUIHandler(self), src = src)
@@ -127,6 +144,8 @@ class GUIPolicy(Policy):
 		assert policy is None
 		policy = self
 
+		self.widgets = Template('main')
+
 		if restrictions:
 			for r in restrictions:
 				self.root_restrictions.append(r)
