@@ -362,16 +362,16 @@ class IfaceCache(object):
 		"""List all interfaces in the cache.
 		@rtype: [str]
 		"""
-		all = {}
+		all = set()
 		for d in basedir.load_cache_paths(config_site, 'interfaces'):
 			for leaf in os.listdir(d):
 				if not leaf.startswith('.'):
-					all[leaf] = True
+					all.add(unescape(leaf))
 		for d in basedir.load_config_paths(config_site, config_prog, 'user_overrides'):
 			for leaf in os.listdir(d):
 				if not leaf.startswith('.'):
-					all[leaf] = True
-		return map(unescape, all.keys())
+					all.add(unescape(leaf))
+		return list(all)	# Why not just return the set?
 
 	def add_to_cache(self, source, data):
 		"""Add an implementation to the cache.
