@@ -157,13 +157,14 @@ class EnvironmentBinding(Binding):
 class Feed(object):
 	"""An interface's feeds are other interfaces whose implementations can also be
 	used as implementations of this interface."""
-	__slots__ = ['uri', 'os', 'machine', 'user_override']
-	def __init__(self, uri, arch, user_override):
+	__slots__ = ['uri', 'os', 'machine', 'user_override', 'langs']
+	def __init__(self, uri, arch, user_override, langs = None):
 		self.uri = uri
 		# This indicates whether the feed comes from the user's overrides
 		# file. If true, writer.py will write it when saving.
 		self.user_override = user_override
 		self.os, self.machine = _split_arch(arch)
+		self.langs = langs
 	
 	def __str__(self):
 		return "<Feed from %s>" % self.uri
@@ -246,11 +247,13 @@ class Implementation(object):
 	"""An Implementation is a package which implements an Interface.
 	@ivar download_sources: list of methods of getting this implementation
 	@type download_sources: [L{RetrievalMethod}]
+	@ivar bindings: how to tell this component where it itself is located (since 0.31)
+	@type bindings: [Binding]
 	"""
 
-	__slots__ = ['upstream_stability', 'user_stability',
+	__slots__ = ['upstream_stability', 'user_stability', 'langs',
 		     'requires', 'main', 'metadata', 'download_sources',
-		     'id', 'interface', 'version', 'released']
+		     'id', 'interface', 'version', 'released', 'bindings']
 
 	def __init__(self, interface, id):
 		assert id
@@ -264,6 +267,8 @@ class Implementation(object):
 		self.version = None
 		self.released = None
 		self.download_sources = []
+		self.langs = None
+		self.bindings = []
 
 	def get_stability(self):
 		return self.user_stability or self.upstream_stability or testing
@@ -300,7 +305,7 @@ class ZeroInstallImplementation(Implementation):
 	"""An implementation where all the information comes from Zero Install.
 	@since: 0.28"""
 	__slots__ = ['os', 'machine', 'upstream_stability', 'user_stability',
-		     'size', 'requires', 'main', 'metadata',
+		     'size', 'requires', 'main',
 		     'id',  'interface']
 
 	def __init__(self, interface, id):
@@ -326,7 +331,8 @@ class Interface(object):
 	"""An Interface represents some contract of behaviour.
 	Note: This class is for both feeds and interfaces. Should really have used separate classes.
 	@ivar uri: the URL for this feed
-	@ivar implementations: list of Implementations in this feed
+	@ivar implementations: Implementations in this feed, indexed by ID
+	@type implementations: {str: L{Implementation}}
 	@ivar name: human-friendly name
 	@ivar summary: short textual description
 	@ivar description: long textual description
@@ -357,7 +363,7 @@ class Interface(object):
 					    "with 'http:'" % uri)
 
 	def reset(self):
-		self.implementations = {}	# Path -> Implementation
+		self.implementations = {}
 		self.name = None
 		self.summary = None
 		self.description = None
