@@ -2,11 +2,11 @@
 Chooses a set of implementations based on a policy.
 """
 
-# Copyright (C) 2006, Thomas Leonard
+# Copyright (C) 2007, Thomas Leonard
 # See the README file for details, or visit http://0install.net.
 
 import time
-import sys, os
+import sys, os, sets
 from logging import info, debug, warn
 import arch
 
@@ -19,6 +19,9 @@ from zeroinstall import NeedDownload
 from zeroinstall.injector.iface_cache import iface_cache, PendingFeed
 from zeroinstall.injector.trust import trust_db
 
+# If we started a check within this period, don't start another one:
+FAILED_CHECK_DELAY = 60 * 60	# 1 Hour
+
 class _Cook:
 	"""A Cook follows a Recipe."""
 	# Maybe we're taking this metaphor too far?
@@ -63,15 +66,15 @@ class _Cook:
 		try:
 			# Unpack each of the downloaded archives into it in turn
 			for step in self.recipe.steps:
-				unpack.unpack_archive(step.url, self.streams[step], tmpdir, step.extract)
+				unpack.unpack_archive_over(step.url, self.streams[step], tmpdir, step.extract)
 			# Check that the result is correct and store it in the cache
 			store.check_manifest_and_rename(self.required_digest, tmpdir)
 			tmpdir = None
 		finally:
 			# If unpacking fails, remove the temporary directory
 			if tmpdir is not None:
-				import shutil
-				shutil.rmtree(tmpdir)
+				from zeroinstall import support
+				support.ro_rmtree(tmpdir)
 
 class Policy(object):
 	"""Chooses a set of implementations based on a policy.
@@ -99,11 +102,13 @@ class Policy(object):
 	@type restrictions: {model.Interface -> [model.Restriction]}
 	@ivar src: whether we are looking for source code
 	@type src: bool
+	@ivar stale_feeds: set of feeds which are present but haven't been checked for a long time
+	@type stale_feeds: set
 	"""
 	__slots__ = ['root', 'implementation', 'watchers',
 		     'help_with_testing', 'network_use',
 		     'freshness', 'ready', 'handler', '_warned_offline',
-		     'restrictions', 'src', 'root_restrictions']
+		     'restrictions', 'src', 'root_restrictions', 'stale_feeds']
 
 	def __init__(self, root, handler = None, src = False):
 		"""
@@ -120,6 +125,7 @@ class Policy(object):
 		self.ready = False
 		self.src = src				# Root impl must be a "src" machine type
 		self.restrictions = {}
+		self.stale_feeds = sets.Set()
 
 		# This is used in is_unusable() to check whether the impl is
 		# for the root interface when looking for source. It is also
@@ -200,14 +206,18 @@ class Policy(object):
 			if not updated:
 				self.handler.confirm_trust_keys(iface, pending.sigs, pending.new_xml)
 
-	def recalculate(self):
+	def recalculate(self, fetch_stale_interfaces = True):
 		"""Try to choose a set of implementations.
 		This may start downloading more interfaces, but will return immediately.
+		@param fetch_stale_interfaces: whether to begin downloading interfaces which are present but haven't
+		been checked within the L{freshness} period
+		@type fetch_stale_interfaces: bool
 		@postcondition: L{ready} indicates whether a possible set of implementations was chosen
 		@note: A policy may be ready before all feeds have been downloaded. As new feeds
 		arrive, the chosen versions may change.
 		"""
 
+		self.stale_feeds = sets.Set()
 		self.restrictions = {}
 		self.implementation = {}
 		self.ready = True
@@ -229,13 +239,19 @@ class Policy(object):
 			if impl:
 				debug("Will use implementation %s (version %s)", impl, impl.get_version())
 				self.implementation[iface] = impl
-				for d in impl.dependencies.values():
+				for d in impl.requires:
 					debug("Considering dependency %s", d)
 					process(d)
 			else:
 				debug("No implementation chould be chosen yet");
 				self.ready = False
-		process(Dependency(self.root, restrictions = self.root_restrictions))
+		process(InterfaceDependency(self.root, restrictions = self.root_restrictions))
+
+		if fetch_stale_interfaces and self.network_use != network_offline:
+			for stale in self.stale_feeds:
+				info("Checking for updates to stale feed %s", stale)
+				self.begin_iface_download(stale, False)
+
 		for w in self.watchers: w()
 	
 	# Only to be called from recalculate, as it is quite slow.
@@ -351,7 +367,7 @@ class Policy(object):
 		@rtype: [model.Implementation]"""
 		impls = iface.implementations.values()
 		for f in self.usable_feeds(iface):
-			feed_iface = self.get_interface(f.uri)
+			feed_iface = iface_cache.get_interface(f.uri)
 			if feed_iface.implementations:
 				impls.extend(feed_iface.implementations.values())
 		impls.sort(lambda a, b: self.compare(iface, a, b))
@@ -394,8 +410,10 @@ class Policy(object):
 		return None
 	
 	def get_interface(self, uri):
-		"""Get an interface from the L{iface_cache}. If it is missing or needs updating,
-		start a new download.
+		"""Get an interface from the L{iface_cache}. If it is missing start a new download.
+		If it is present but stale, add it to L{stale_feeds}. This should only be called
+		from L{recalculate}.
+		@see: iface_cache.iface_cache.get_interface
 		@rtype: L{model.Interface}"""
 		iface = iface_cache.get_interface(uri)
 
@@ -419,12 +437,16 @@ class Policy(object):
 							"(so not fetching).", uri)
 						self._warned_offline = True
 		elif not uri.startswith('/'):
-			staleness = time.time() - (iface.last_checked or 0)
+			now = time.time()
+			staleness = now - (iface.last_checked or 0)
 			debug("Staleness for %s is %.2f hours", iface, staleness / 3600.0)
 
-			if self.network_use != network_offline and self.freshness > 0 and staleness > self.freshness:
-				debug("Updating %s", iface)
-				self.begin_iface_download(iface, False)
+			if self.freshness > 0 and staleness > self.freshness:
+				if iface.last_check_attempt and iface.last_check_attempt > now - FAILED_CHECK_DELAY:
+					debug("Stale, but tried to check recently (%s) so not rechecking now.", time.ctime(iface.last_check_attempt))
+				else:
+					debug("Adding %s to stale set", iface)
+					self.stale_feeds.add(iface)
 		#else: debug("Local interface, so not checking staleness.")
 
 		return iface
@@ -442,7 +464,7 @@ class Policy(object):
 			# Possibly we should handle this better, but it's unlikely anyone will need
 			# to use an interface as an icon or implementation as well, and some of the code
 			# assumes it's OK keep asking for the same interface to be downloaded.
-			info("Already have a handler for %s; not adding another", interface)
+			debug("Already have a handler for %s; not adding another", interface)
 			return
 
 		def feed_downloaded(stream):
@@ -567,6 +589,8 @@ class Policy(object):
 		@type impl: model.Implementation
 		@rtype: bool
 		"""
+		if isinstance(impl, DistributionImplementation):
+			return impl.installed
 		if impl.id.startswith('/'):
 			return os.path.exists(impl.id)
 		else:
@@ -584,7 +608,7 @@ class Policy(object):
 	
 	def get_uncached_implementations(self):
 		"""List all chosen implementations which aren't yet available locally.
-		@rtype: [model.Implementation]"""
+		@rtype: [(str, model.Implementation)]"""
 		uncached = []
 		for iface in self.implementation:
 			impl = self.implementation[iface]
@@ -599,7 +623,7 @@ class Policy(object):
 		for x in self.implementation:
 			self.begin_iface_download(x, force)
 			for f in self.usable_feeds(x):
-				feed_iface = self.get_interface(f.uri)
+				feed_iface = iface_cache.get_interface(f.uri)
 				self.begin_iface_download(feed_iface, force)
 	
 	def interface_changed(self, interface):
@@ -612,7 +636,8 @@ class Policy(object):
 		This is used by B{0launch --feed}.
 		@rtype: [model.Interface]
 		@raise SafeException: If there are no known feeds."""
-		feed_iface = self.get_interface(feed_iface_uri)
+		# TODO: what if it isn't cached yet?
+		feed_iface = iface_cache.get_interface(feed_iface_uri)
 		if not feed_iface.feed_for:
 			if not feed_iface.name:
 				raise SafeException("Can't get feed targets for '%s'; failed to load interface." %
@@ -620,9 +645,10 @@ class Policy(object):
 			raise SafeException("Missing <feed-for> element in '%s'; "
 					"this interface can't be used as a feed." % feed_iface_uri)
 		feed_targets = feed_iface.feed_for
+		debug("Feed targets: %s", feed_targets)
 		if not feed_iface.name:
 			warn("Warning: unknown interface '%s'" % feed_iface_uri)
-		return [self.get_interface(uri) for uri in feed_targets]
+		return [iface_cache.get_interface(uri) for uri in feed_targets]
 	
 	def get_icon_path(self, iface):
 		"""Get an icon for this interface. If the icon is in the cache, use that.
