@@ -25,30 +25,38 @@ class InvalidInterface(SafeException):
 			message += "\n\n(exact error: %s)" % ex
 		SafeException.__init__(self, message)
 
+_binding_names = frozenset(['environment'])
+
+def _process_binding(e):
+	if e.name == 'environment':
+		mode = {
+			None: EnvironmentBinding.PREPEND,
+			'prepend': EnvironmentBinding.PREPEND,
+			'append': EnvironmentBinding.APPEND,
+			'replace': EnvironmentBinding.REPLACE,
+		}[e.getAttribute('mode')]
+			
+		binding = EnvironmentBinding(e.getAttribute('name'),
+					     insert = e.getAttribute('insert'),
+					     default = e.getAttribute('default'),
+					     mode = mode)
+		if not binding.name: raise InvalidInterface("Missing 'name' in binding")
+		if binding.insert is None: raise InvalidInterface("Missing 'insert' in binding")
+		return binding
+	else:
+		raise Exception("Unknown binding type '%s'" % e.name)
+
 def _process_depends(item):
 	# Note: also called from selections
 	dep_iface = item.getAttribute('interface')
-	if dep_iface is None:
+	if not dep_iface:
 		raise InvalidInterface("Missing 'interface' on <requires>")
 	dependency = InterfaceDependency(dep_iface, metadata = item.attrs)
 
 	for e in item.childNodes:
 		if e.uri != XMLNS_IFACE: continue
-		if e.name == 'environment':
-			mode = {
-				None: EnvironmentBinding.PREPEND,
-				'prepend': EnvironmentBinding.PREPEND,
-				'append': EnvironmentBinding.APPEND,
-				'replace': EnvironmentBinding.REPLACE,
-			}[e.getAttribute('mode')]
-				
-			binding = EnvironmentBinding(e.getAttribute('name'),
-						     insert = e.getAttribute('insert'),
-						     default = e.getAttribute('default'),
-						     mode = mode)
-			if not binding.name: raise InvalidInterface("Missing 'name' in binding")
-			if binding.insert is None: raise InvalidInterface("Missing 'insert' in binding")
-			dependency.bindings.append(binding)
+		if e.name in _binding_names:
+			dependency.bindings.append(_process_binding(e))
 		elif e.name == 'version':
 			dependency.restrictions.append(
 				Restriction(not_before = parse_version(e.getAttribute('not-before')),
@@ -134,7 +142,7 @@ def update_user_overrides(interface):
 			feed_src = item.getAttribute('src')
 			if not feed_src:
 				raise InvalidInterface('Missing "src" attribute in <feed>')
-			interface.feeds.append(Feed(feed_src, item.getAttribute('arch'), True))
+			interface.feeds.append(Feed(feed_src, item.getAttribute('arch'), True, langs = item.getAttribute('langs')))
 
 def check_readable(interface_uri, source):
 	"""Test whether an interface file is valid.
@@ -208,6 +216,10 @@ def update(interface, source, local = False):
 
 	try:
 		root = qdom.parse(file(source))
+	except IOError, ex:
+		if ex.errno == 2:
+			raise InvalidInterface("Feed not found. Perhaps this is a local feed that no longer exists? You can remove it from the list of feeds in that case.", ex)
+		raise InvalidInterface("Can't read file", ex)
 	except Exception, ex:
 		raise InvalidInterface("Invalid XML", ex)
 
@@ -267,34 +279,48 @@ def update(interface, source, local = False):
 			if not feed_src:
 				raise InvalidInterface('Missing "src" attribute in <feed>')
 			if feed_src.startswith('http:') or local:
-				interface.feeds.append(Feed(feed_src, x.getAttribute('arch'), False))
+				interface.feeds.append(Feed(feed_src, x.getAttribute('arch'), False, langs = x.getAttribute('langs')))
 			else:
 				raise InvalidInterface("Invalid feed URL '%s'" % feed_src)
 		else:
 			interface.add_metadata(x)
 
-	def process_group(group, group_attrs, base_depends):
+	def process_group(group, group_attrs, base_depends, base_bindings):
 		for item in group.childNodes:
 			if item.uri != XMLNS_IFACE: continue
 
+			if item.name not in ('group', 'implementation', 'package-implementation'):
+				continue
+
 			depends = base_depends[:]
+			bindings = base_bindings[:]
 
 			item_attrs = _merge_attrs(group_attrs, item)
 
+			# We've found a group or implementation. Scan for dependencies
+			# and bindings. Doing this here means that:
+			# - We can share the code for groups and implementations here.
+			# - The order doesn't matter, because these get processed first.
+			# A side-effect is that the document root cannot contain
+			# these.
 			for child in item.childNodes:
 				if child.uri != XMLNS_IFACE: continue
 				if child.name == 'requires':
 					dep = _process_depends(child)
 					depends.append(dep)
+				elif child.name in _binding_names:
+					bindings.append(_process_binding(child))
 
 			if item.name == 'group':
-				process_group(item, item_attrs, depends)
+				process_group(item, item_attrs, depends, bindings)
 			elif item.name == 'implementation':
-				process_impl(item, item_attrs, depends)
+				process_impl(item, item_attrs, depends, bindings)
 			elif item.name == 'package-implementation':
 				process_native_impl(item, item_attrs, depends)
+			else:
+				assert 0
 
-	def process_impl(item, item_attrs, depends):
+	def process_impl(item, item_attrs, depends, bindings):
 		id = item.getAttribute('id')
 		if id is None:
 			raise InvalidInterface("Missing 'id' attribute on %s" % item)
@@ -326,6 +352,7 @@ def update(interface, source, local = False):
 		impl.main = item_main
 
 		impl.released = item_attrs.get('released', None)
+		impl.langs = item_attrs.get('langs', None)
 
 		size = item.getAttribute('size')
 		if size:
@@ -342,6 +369,7 @@ def update(interface, source, local = False):
 			raise InvalidInterface("Upstream can't set stability to preferred!")
 		impl.upstream_stability = stability
 
+		impl.bindings = bindings
 		impl.requires = depends
 
 		for elem in item.childNodes:
@@ -405,4 +433,4 @@ def update(interface, source, local = False):
 		{'stability': 'testing',
 	         'main' : root.getAttribute('main') or None,
 		},
-		[])
+		[], [])
